﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace OpenDLM
{
    // <cc:license>http://creativecommons.org/licenses/by-nc-sa/3.0/</cc:license>
    // <cc:license>http://creativecommons.org/licenses/by-nd/3.0/</cc:license>

    class CCLicense
    {
        public String area = null, version = "3.0";
        public bool by = false, nc = false, nd = false, sa = false;

        public CCLicense() {}
        public override String ToString()
        {
            String license = "";
            if (by) license += " by";
            if (nc) license += " nc";
            if (sa) license += " sa";
            if (nd) license += " nd";
            license = license.Trim().Replace(' ', '-');
            license += "/"+version+"/";
            if (area != null) license += area+"/";
            return "<cc:license>http://creativecommons.org/licenses/" + license + "</cc:license>";
        }
    }

    class MetaFile
    {
        static Encoding encoder = Encoding.GetEncoding("UTF-8", new EncoderExceptionFallback(), new DecoderExceptionFallback());

        static void Main(string[] args)
        {
            CCLicense license = new CCLicense();
            license.by = true; license.sa = true; license.nc = true;
            String metadata = license.ToString();
            createMetaFile("testdata/opensource.pdf", metadata);
            MetaFile metaFile = new MetaFile("testdata/opensource.pdf.meta");
            String metaheader = metaFile.parseMetaHeader();
            Console.WriteLine(metaheader);
            metaFile.saveSourceFile("testdata/opensource.pdf.meta.pdf");
            metaFile.close();
        }

        static void createMetaFile(String pFileName, String pMetadata)
        {
            FileStream toFile = File.Open(pFileName + ".meta", FileMode.Create);
            byte[] metaBytes = encoder.GetBytes("<metadata>\n" + pMetadata + "\n</metadata>\n");
            toFile.Write(metaBytes, 0, metaBytes.Length);
            FileStream sourceFile = File.Open(pFileName, FileMode.Open);
            byte[] buffer = new byte[1024];
            while (true)
            {
                int len = sourceFile.Read(buffer, 0, buffer.Length);
                toFile.Write(buffer, 0, len);
                if (len < buffer.Length) break;
            }
            toFile.Close();
        }

        FileStream metaFile;
        BinaryReader metaReader;

        public MetaFile(String pMetaFileName) {
            metaFile = File.Open(pMetaFileName, FileMode.Open);
            metaReader = new BinaryReader(metaFile);
        }

        String parseMetaHeader()
        {
            StringBuilder metadata = new StringBuilder();
            while (true)
            {
                char ch = metaReader.ReadChar();
                metadata.Append(ch);
                if (metadata.ToString().EndsWith("</metadata>\n")) break;
            }
            return metadata.ToString();
        }

        void saveSourceFile(String toSourceFileName)
        {
            FileStream sourceFile = File.Open(toSourceFileName, FileMode.Create);
            byte[] buffer = new byte[1024];
            while (true)
            {
                int len = metaReader.Read(buffer, 0, buffer.Length);
                sourceFile.Write(buffer, 0, len);
                if (len < buffer.Length) break;
            }
            sourceFile.Close();
        }

        void close() { metaFile.Close(); }
    }

    class OpenDLM
    {
    }
}
