using System;
using System.Collections.Generic;
using System.Text;

namespace OpenCL
{
    class CountMap : Dictionary<String, int>
    {
        public void add(String str)
        {
            if (ContainsKey(str))
                this[str]++;
            else
                this.Add(str, 1);
        }

        public void print(int countMin)
        {
            foreach (KeyValuePair<String, int> pair in this)
                if (pair.Value >= countMin)
                    Debug.print(pair.ToString());
        }
    }

    class WordLearner
    {
        public CountMap countMap = new CountMap();

        public static void Main(String[] args)
        {
            String corpus = TextFile.fileToText("corpus/parallel1500.txt");
            WordLearner learner = new WordLearner();
            learner.run(corpus);
            List<String> words = learner.getWordList();
            Debug.print(words);
        }

        public WordLearner() {}

        public void run(String corpus)
        {
            count(corpus, countMap);
        }

        public void count(String text, CountMap countMap)
        {
            for (int i = 0; i < text.Length; i++)
            {
                String substr = text.Substring(i, Math.Min(text.Length - i, 6));
                for (int len = 2; len < substr.Length; len++)
                {
                    char ch = substr[len-1];
                    if (ch >= 0 && ch <= 127 || STR.ChineseSymbols.IndexOf(ch) >= 0)
                        break;
                    String head = substr.Substring(0, len);
                    countMap.add(head);
                }
            }
        }

        public List<String> getWordList()
        {
            List<String> words = new List<String>();
            CountMap rfreeMap = new CountMap();
            CountMap lfreeMap = new CountMap();
            foreach (String str in countMap.Keys)
            {
                String head = str.Substring(0, str.Length-1);
                String tail = str.Substring(1, str.Length-1);
                rfreeMap.add(head);
                lfreeMap.add(tail);
            }
            foreach (String str in countMap.Keys)
            {
                if (rfreeMap.ContainsKey(str) && lfreeMap.ContainsKey(str))
                    if (rfreeMap[str] >= 2 && lfreeMap[str] >= 2)
                        words.Add(lfreeMap[str] + "(" + str + ")" + rfreeMap[str]);
            }
            return words;
        }
    }
}